#/bin/sh
set -x
#
#  This script generates a grib1 file for a grib2 file.
#  Using an alert for the grib2 file and the grib2 filename, a
#  new name is created using the name generator. It then uses 
#  cnvgrib to create the new file, and it is alerted with the
#  the new type/subtype.
#
#  Arguments:
#
#  alert for grib2 file
#  grib1 filename
#  type
#  subtype
#



if [ $# -ne 4 ]
then
  echo "$0 alert grib2_path type subtype"
  exit 1;
fi

grib2_path=$1
alert=$2
type=$3
subtype=$4

#  Get the new path using the dbn_name_generator

grib1_path=`$DBNROOT/bin/dbn_name.pl $grib2_path $alert`
status=$?
if [ $status -ne 0 ]; then
   echo Cannot find grib1 filename from dbn_name for: $grib2_path $alert
   exit
fi

#  Determine and create new directory

grib1_dir=`dirname $grib1_path`
grib1_file=`basename $grib1_path`

if [ ! -d $grib1_dir ] ; then

   mkdir -pm 775 $grib1_dir
   status=$?

   if [ $status -ne 0 ]
   then
     echo "Error creating directory $grib1_dir, $?"
     exit $status
   fi
fi

#  Create grib1 file as a dot file, if cnvgrib is successful do a mv

tmp_grib1_file=.$grib1_file 
tmp_grib1_path=$grib1_dir/$tmp_grib1_file 
grib1_path=$grib1_dir/$grib1_file 

$DBNROOT/bin/cnvgrib -g21 $grib2_path $tmp_grib1_path

status=$?

if [ $status -ne 0 ]
then
   echo "Problem with cnvgrib, exiting with $status"
   exit $status
fi

mv $tmp_grib1_path $grib1_path

$DBNROOT/bin/dbn_alert $type $subtype convert_grib2to1.sh $grib1_path

