#!/usr/bin/perl
#
# This script will generate an grib inventory file (idx) for
# The input grib1 file. If all files if the set are in, 
# create a combined ctl file and idx file.
#
#
#
use Time::CTime;           # for strftime

sub usage;

$nargs = @ARGV;
if ($nargs < 3)  {
   usage ();
}

$grib_filename=shift;
$filelist_template=shift;
$make_combined_ctl_script=shift;
$opt_arg1=shift;
$opt_arg2=shift;
print "args: $opt_arg1 $opt_arg2\n";

# Get date and cycle from filename

$grib_filename =~ /(.*)\/.*/;
$dirname=$1;
print "dirname=$dirname\n";
print "grib_filename=$grib_filename\n";
$grib_filename =~ /.*\/.*(\d{8})\/.*\.t(\d\d)z\..*/;
$yyyymmdd=$1;
$cycle=$2;
print "yyyymmdd=$yyyymmdd cycle=$cycle\n";


$dbn_root = $ENV{'DBNROOT'};
$data_root = $ENV{'DATAROOT'};

$today=strftime("%Y%m%d", gmtime(time));
$combined_ctl_log="$dbn_root/log/${make_combined_ctl_script}.log.$today";

# Make the ctl, idx, and inv files

if (make_files( $grib_filename ) ) {

#  If we have all the files, make the combined ctl file.

if ( $filelist_template =~ /^none$/) {
       print "no template, just call $dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname\n";
      `sh -x $dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname >> $combined_ctl_log 2>&1`

} else {

   if (check_for_file_list ($filelist_template, $yyyymmdd, $cycle, $opt_arg1, $opt_arg2)) {
       print "call $dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname $opt_arg1 $opt_arg2\n";
   
      `sh -x $dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname $opt_arg1 $opt_arg2 >> $combined_ctl_log 2>&1`
   }
}

} else {
   print "make_files returned fals\n";
}


sub check_for_file_list() {

######################################################################
# Purpose:
#  Check that files are present on local host.  
#  If all are present, return 0.  If any are missing, return 1.
#  
#
# Input:
#       fileset table: file name containing a directory name and filenames
#
# Author: Paula Freeman, some cobbled from Richard Putt's script.
######################################################################

   
   $verbose=1;
   
   $fileTable= shift;
   $YYYYMMDD = shift;
   $CC = shift;
   $ARG1= shift;
   $ARG2= shift;
   
print "fileTable = $fileTable\n";
print "YYYYMMDD = $YYYYMMDD\n";
print "CC  = $CC\n";
print "ARG1= $ARG1\n";
print "ARG2= $ARG2\n";

   # Define hash table
   
   %fileListHash=();

    if ( $fileTable =~ /RUC/) {

    $off_hr= $CC % 3;
    if ( $off_hr != 0 ) {
       $fileTable = "nomad_RUC2_offhr_template";
    } else {
       $fileTable = "nomad_RUC2_3hr_template"; 
    }
    }
    

   
   # Open the table
   
   if (! open (TABLE,"${dbn_root}/table/nomad/files_templates/${fileTable}"))  {
      print "$0: Cannot open $fileTable: $!";
      exit 0;
   }
   
   # First line is the directory pattern
   
   $baseDir=<TABLE>;
   chomp $baseDir;
   $baseDir =~ s/YYYYMMDD/$YYYYMMDD/g;
   $baseDir =~ s/CC/$CC/g;
   if ($ARG1 !~ /^$/) {
      $baseDir =~ s/ARG1/$ARG1/g;
   }
   if ($ARG2 !~ /^$/) {
      $baseDir =~ s/ARG2/$ARG2/g;
   }
 
   $verbose && print "\nChecking for files in $baseDir \n";
   
   # The rest of the lines are the filenames.
   
   while ($filename=<TABLE>) {
      chomp $filename;
      $filename =~ s/CC/$CC/g;
      if ($ARG1 !~ /^$/) {
         $filename =~ s/ARG1/$ARG1/g;
      }
      if ($ARG2 !~ /^$/) {
         $filename =~ s/ARG2/$ARG2/g;
      }

      $fileListHash{$filename} = "1";
   }
   
   #foreach $key (keys %fileListHash) {
   #   print "$key\n";
   #}
   
   
   # Read the directory.
   # If directory doesn't exist, and it's still before the 
   # directory creation time limit, just exit.  If after
   # then generate and error.
   
   if (!opendir(DIR, $baseDir)) {
      print "$0: opendir($baseDir) failed: $!\n";
      exit 1;
   }
   @dirLines=readdir(DIR);
   closedir(DIR);
   
   foreach $f (@dirLines){
      chomp $f;
      $filePath = "$baseDir/$f";
      # next if !-f $filePath;
     if ( exists $fileListHash{$f} ){
       delete $fileListHash{$f} ;
     } else {
   #    print "WARNING: File not in table " . $hostname . ":" . $baseDir . "$filePath\n";
     }
   }
   
   
   # Print out names of missing files
   
   @missingFiles = sort keys %fileListHash;
   $missedFileCount = scalar( @missingFiles );
   
   #print "number of missing files: $missedFileCount\n";
   if ( $missedFileCount != 0) {
      print "\nFiles that are missing: \n\n";
      for $file ( sort keys( %fileListHash ) ){
        print "$baseDir/$file\n";
      }
      return 0;
   } else {
      $verbose && print "\nGOOD: No missing/late files.\n";
      return 1;
   }
   
}
   
sub usage {
      print "usage: $0: grib_filename file_list_template make_ctl_script [optional arg]\n";
   cleanup;
   exit;
}

sub make_files { 

   print "in make_files\n";
   $grib_filename = shift;
   
   $today=strftime("%Y%m%d", gmtime(time));
   $inv_log="$dbn_root/log/inv.log.$today";
   $ctl_log="$dbn_root/log/ctl.log.$today";
   $idx_log="$dbn_root/log/idx.log.$today";
   
   $wgrib="$dbn_root/bin/wgrib";
   $grib2ctl="$dbn_root/user/grib2ctl.pl";
   $gribmap="/usr/local/bin/gribmap";
   
   $grib_inv_file = $grib_filename . ".inv";
#  $grib_ctl_file = $grib_filename . ".ctl";
#  $grib_idx_file = $grib_filename . ".idx";
   
   print "Generate inv file $grib_inv_file\n";
   print "command: $wgrib $grib_filename -s  > $grib_inv_file\n" ;
   `echo "Generate inv file $grib_inv_file"  >> $inv_log `;
   `$wgrib $grib_filename -s > $grib_inv_file 2>> $inv_log `;
   if ( -z $grib_inv_file) {
      `echo "Error making inv file for $grib_file"  >> $ctl_log`;
      unlink $grib_inv_file;
   }
   
#  print "Generate ctl file $grib_ctl_file\n";
#  `echo "Generate ctl file $grib_ctl_file"  >> $ctl_log`;
#  `$grib2ctl $grib_filename > $grib_ctl_file 2>> $ctl_log `;
#  if ( -z $grib_ctl_file) {
#     `echo "Error making ctl file $grib_ctl_file"  >> $ctl_log `;
#  } else {
#     print "Generate idx file $grib_idx_file\n";
#     `echo "Generate idx file $grib_idx_file"  >> $idx_log `;
#     `$gribmap -e -i $grib_ctl_file  >> $idx_log 2>&1 `;
#     if ( -z "${grib_file}.idx") {
#        `echo "Error making idx file for $grib_ctl_file"  >> $idx_log`;
#        return 0;
#     }
#  }
return 1 ;
}

